.subckt behavioral_input 1 2 3 6 7 8 out

* isolate inner sections of receiver model from external pins
eina in 0 v='v(2,3)'
einb inn 0 v='v(8,3)'

* eina in 0 v='v(2,3)*(1-v(5,3))'
* this disables the receiver when the driver is tri-state when
* combined into an IO cell

epwr power 0 v='v(1,3)'

evref vref 0 v='v(inn,0)'
* if the Rx uses as static Vref instead of comparing with 
* the other differential signal to switch, then comment
* the line above and uncomment the next line OR feed the
* static Vref in on pin 8 and leave as is
* Vvref vref 0   1.0
* hard-code vref ^--- here

xdiffamp power in vref 0 out bhvr_in

* receiver output capacitance
Cout out 0 20ff

* prints/names waveforms for display with unique names
*.node_param out name=(name(2).rcvrout) print
* but this is printed automatically by the EM A_to_D

*****************************************************
*         BEHAVIORAL INPUT SUBCIRCUIT               *
*****************************************************

.subckt bhvr_in   1      2    vref    3     out
*                power input  vref  ground output

* this is the low-pass filter capacitance value
.param c_filt='1.5p'

* subckt for amplifier - assuming vref = voltage of node ref
* first generate vcritcial.  This is the Actual receiver DC switching point

* This is the DC transfer curves as a function of Vref
ecrit crit 3 pwl lpf2 3
*    outputs     inputs
* pwl=type of source

datapoints vv
* vref  critical switch voltage  (columns)
0.5 0.7
1.0 1.0
1.5 1.3
end vv

* Low Pass Input Stage
* The capacitance is used to 
* compensate for delay characteristics
* with respect to Vref.  The front-end 
* LPF filters out spikes...at Vref=1.0
* the spike width rejected is 100 ps.

Cfilter lpf 3 'c_filt'
Rin 2 lpf 100
Rout lpf 3 1e6

* i = Vr/R, R=100.  Scale by 1e10, R*Scale = 1e12

* THE NEXT LINE SOLVES DV/DT OF THE INPUT SLOPE!!!!
* dV/dt = V/R*C
ederiv deriv 3 v='v(2,lpf)/(100*1e10*'c_filt')'

Rref vref lpf2 100
Cref lpf2 3 1.7p   
Rout lpf2 3 1e6

eout 6 3 pwl lpf crit
* slews the transfer curve to 80mV/2.0V

datapoints vv
5.0    2.0
0.040  2.0
-0.040 0
-5.0   0
end vv

* This delay line adds delay based on the rise/fall edge rate in the
* transition region (+/- 150 mV).  This region is of arbitrary size
* and may need to be changed for future refinement.

* ---------------------------Extra Delay Line ----------------------

x1 deriv lpf crit 6 8 lpf2 3 delay_adder

* .node_param 8 name=(name(2).int8) print

Cdamp 8 3 0.1pf
*-------------------------------------------------------------------

* Now add the regular intrinsic delay (the delay from a 100 ps edge)
* Get this from testing circuit with the fastest edge speed you 
* expect (100pS in this case).
* This is a special e source, a threshold controlled voltage source
* with a time delay in the left column.

* Note that the output voltage of the receiver is set here to 2V

e out 3 pwl 8 3

datapoints timecoeff vlohi=1.0 vhilo=1.0
rising
0 0
.020n 0
.090n 2.0
falling
0     2.0
.110n 2.0
.190n 0
end timecoeff

*---------------------------Delay Adder Subcircuit Def ---------------

.subckt delay_adder deriv1a filtered_in crit delayin out vref 3

.param cderiv 1.0

* If dV/dt < 10.0V/ns and > 0 (rising edge) add up to 40-60 ps of 
* delay, if
* dV/dt < 0, then add up to 50-60 ps of delay, otherwise add 1ps (~ 
* 0).

* table is now directly in terms of dly
* Check for transition region.  If not in the transition region
* do no delay adjustments.

* Hack for numerical integration oscillations

ederiv1 deriv1 0 v='(v(deriv1a)+prev(v(deriv1a)))/2' 

* the extra delay is a function of 100mV either side of vref, it may
* cost you up to 60pS if you are slow

edlylvla dlylvl 0 v='if((abs(v(filtered_in,crit)) < 0.1)) (v(deriv1)) else (3)'

edly dly 0 pwl dlylvl 0 

datapoints vv
-3.0 0.001n
-2.0 0.001n
-1.0 0.005n
-.2  0.005n
-.1  0.020n
-.04 0.050n
0.0 0.060n
.04 0.040n
.1   0.015n
.2   0.005n
1.0  0.005n
2.0  0.001n
3.0  0.001n
end vv

x delayin 31 dly 0 vardlysec sec=5
x 31 32 dly 0 vardlysec sec=5
x 32 33 dly 0 vardlysec sec=5
x 33 34 dly 0 vardlysec sec=5
x 34 out dly 0 vardlysec sec=5

* Resistor added to eliminate reflections

Rout out 0 50
.ends delay_adder

.subckt vardlysec delayin out dly ref imp=50 sec=5

ecap cap 0 v='v(dly)/(imp * sec)'

eind ind 0 v='imp * imp * v(cap)'

vind delayin in1 0

* no charge conservation
*            V=   L    *  di / dt
eind in1 out v='v(ind) * ddt(i(vind))'
*            I    C    *  dv / dt
gcap out ref i='v(cap) * ddt(v(out,ref))'

.ends vardlysec

.ends bhvr_in

.ends behavioral_input
