.subckt spice_2tap nvdd outp ngnd in en  pcl  gcl  outn  BUFF=BUFF ibis_file=ibis_models.inc 
+ bitp=400p
+ inv0=0
+ inv1=1
+ scale=1
+ padcap=1p
+ cf0=1
+ cf1=1
+ rt=50

*  Some parameters are derived for use in the MacroModel.

*  Some voltage sources are defined to drive the MacroModel. This uses an E element,
*  which is a VCVS, or voltage controlled voltage source.
*
*  N side input voltage to differential buffer
einn inn ngnd v='1 - v(in, ngnd)'

*  The node_param statement tells tlsim to save this waveform for SigWave display.
*  Using the special syntax below, tlsim automatically increments a unique name suffix.
*  Commented out here as an example.
*   .node_param in name=(name(outp) stim) print
* 
*  This is a 2 tap diff driver. You can tell this from the fact that there are
*  2 cf parameters for coefficients and 2 inv parameters for current direction.
*
*  P side driver subcircuit call
xp nvdd outp ngnd in en txpre_2 BUFF=BUFF ibis_file=ibis_file bitp=bitp inv0=inv0 inv1=inv1
+ scale=scale
+ cf0=cf0
+ cf1=cf1
+ rt=rt
+ padcap=padcap

*  N side driver subcircuit call
xn nvdd outn ngnd inn en txpre_2 BUFF=BUFF ibis_file=ibis_file bitp=bitp inv0=inv0 inv1=inv1 
+ scale=scale
+ cf0=cf0
+ cf1=cf1
+ rt=rt
+ padcap=padcap

*  Here is the main single-ended driver subcircuit, called txpre_2.
.subckt txpre_2 nvdd out ngnd in en 
+ bitp=400p
+ inv0=0
+ inv1=1
+ cf0=1
+ cf1=0.1
+ scale=2
+ padcap=1p
+ rt=50

*  Internal termination resistor and pad capacitance elements.
r nvdd out 'rt'
c out ngnd 'padcap'

*  Here are the subcircuit calls for the tap inputs, using subcircuit delayin.
xin0 in0 in ngnd delayin inv=inv0 
xin1 in1 in ngnd delayin inv=inv1 del='bitp'

*  Here are the calls for the tx subcircuits, which contain the taps.
xtx0 nvdd out ngnd in0 en tx scale='scale * cf0'
xtx1 nvdd out ngnd in1 en tx scale='scale * cf1'

*  This is the subcircuit definition for tx, used for the taps.
.subckt tx nvdd out ngnd in en scale=1
* if there is a problem using passing the ibis_file, use
* bdrvr nvdd out ngnd in en Model=2tap_data File=2tap_model_data.inc c_compX=0  VIScale_pulldown='scale'  VIScale_groundclamp=0
bdrvr nvdd out ngnd in en Model=BUFF File=ibis_file c_compX=0  VIScale_pulldown='scale'  VIScale_groundclamp=0
.ends tx

*  The delayin subcircuit drives the inputs to the tx subcircuit.
.subckt delayin in1 in ngnd inv=0 del=0
einp in2 ngnd v='inv * (1 - v(in,ngnd)) + (1-inv) * v(in,ngnd)'
ein1 in1 ngnd pwl in2 ngnd delay=del
datapoints vv
0 0
1 1
end vv
.ends delayin

.ends txpre_2

.ends spice_2tap
