.subckt cal_out nvdd out ngnd in en pcl gcl BUFF=BUFF ibis_file=ibis_models.inc 
+ dvc=0.4 
+ Rref=50 
+ scale=1
+ fudge=1

* setup reference driver
vinref inref ngnd  0
venref enref ngnd 1
rref ref nvdd 'Rref'

* setup drivers pullup resistor
rt out nvdd 'Rref'

* for this circuit to work correctly, you must hold the driver on (low) for 30nS prior to switching
* this allows the driver model time to sense the interconnect and complete self-calibration

* reference driver to determine start impedance
bdrvr nvdd ref ngnd inref enref Model=BUFF File=ibis_file c_compX=0  VIScale_groundclamp=0 VIScale_pulldown='scale'

* compute the starting conductance of the driver using a reference load
egref gref 0 v='(v(nvdd, ref)/Rref)/(1e-12 + v(ref, ngnd))'

* sample and hold initial stabilized interconnect voltage
eoutref outref ngnd v='if (time < 10n) (v(out,ngnd)) else (prev (v(outref, ngnd)))'

* compute unknown conductance of the interconnect
egtot gtot 0 v='(v(gref) * v(outref, ngnd))/(1e-12 + v(nvdd, outref))'

* compute desired output voltage from current voltage and desired swing
evdes nvdes ngnd v='v(nvdd, ngnd) - dvc'

* compute desired output conductance for this interconnect to acheive desired voltage
egdes gdes 0 v='v(gtot) * dvc/(1e-6 + v(nvdes, ngnd))'

* compute and lock down pulldown multiplier between 20n and 25n
epdnXf pdnXf ngnd v='if (time < 20n) (1.0) elseif (time >= 20n && time < 25n) (fudge * v(gdes)/v(gref)) else (prev (v(pdnXf)))'
epdnX pdnX ngnd v='scale * v(pdnXf)'
* use scale and fudge to do what they say, if needed

* to look at the interesting parameters
.node_param pdnXf name=pdnXf print
.node_param gref name=gref print
.node_param gdes name=gdes print

* call actual driver with derived multiplier for pulldown
xout nvdd out ngnd in en pcl gcl pdnX tx

.subckt tx nvdd out ngnd in en pcl gcl pdnX 

* do not scale pullup or clamps
vpupx pupX ngnd 0
vpclX pclX ngnd 0
vgclX gclX ngnd 0

* scale the pulldown by the amount pndX
bdrvr nvdd out ngnd in en pcl gcl pupX pdnX pclX gclX Model=BUFF File=ibis_file c_compX=0  VIScale_groundclamp=0
.ends tx

.ends cal_out